

#include <stddef.h>
#include <assert.h>
#include "uclock.h"
#include "vesa20.h"


uclock_t usec_clock(Uclock_T *now)
{
      static int init = 0;
      Uclock_T buf, *ptr;

      if (NULL == now)
            ptr = &buf;
      else  ptr = now;

      if (!init)                          /* Only reprogram CTC once!   */
      {
            SetMode2_();
            init = -1;
      }

      Uclock_(ptr);

      /*
      ** If Win or OS/2, ticks may lag CTC rollover so wait for
      ** monotonically increasing readings.
      */

      if (env_type)
      {
            static Uclock_T last;

            while (1)
            {
                  if ((last.ticks > ptr->ticks) ||
                        ((last.ticks == ptr->ticks) &&
                        (last.count > ptr->count)))
                  {
                        ++(ptr->ticks);
                        last.ticks = ptr->ticks;
                        last.count = ptr->count;
                        Uclock_(ptr);
                  }
                  else
                  {
                        last.ticks = ptr->ticks;
                        last.count = ptr->count;
                        break;
                  }
            }
      }

      return ((NULL == now) ? uclock_cnvrt(ptr) : -1.0);
}


uclock_t uclock_diff(Uclock_T *Start, Uclock_T *Stop, Uclock_T *Diff)
{
      Uclock_T buf, *ptr = ((NULL == Diff) ? &buf : Diff);

      /*
      ** Add rollover constant if the day has changed
      */

      if (Start->ticks <= Stop->ticks)
            ptr->ticks = Stop->ticks - Start->ticks;
      else  ptr->ticks = Stop->ticks + 0x001800B0L - Start->ticks;

      ptr->count = Stop->count - Start->count;
      if (Stop->count < Start->count)
            --(ptr->ticks);

      return ((NULL == Diff) ? uclock_cnvrt(ptr) : -1.0);
}


uclock_t uclock_cnvrt(Uclock_T *ticks)
{
      return (uclock_t)(ticks->count) + (uclock_t)(65536.0 * ticks->ticks);
}


void usec_delay(unsigned long usecs)
{
      Uclock_T tm0, tm1, diff;
      unsigned long et;

      assert(usecs < 40904449L);          /* Avoid 40.9 sec. overflow */

      Uclock_(&tm0);
      do
      {
            Uclock_(&tm1);
            uclock_diff(&tm0, &tm1, &diff);

            /*
            ** Convert ticks to uSecs. using integer math
            */

            et = CAST(unsigned long, diff);

            et *= 88L;
            et /= 105L;
      } while (usecs > et);
}

