/*** Z80Em: Portable Z80 emulator *******************************************/
/***                                                                      ***/
/***                                Z80IO.h                               ***/
/***                                                                      ***/
/*** This file contains the prototypes for the functions accessing memory ***/
/*** and I/O                                                              ***/
/***                                                                      ***/
/*** Copyright (C) Marcel de Kogel 1996,1997                              ***/
/***     You are not allowed to distribute this software commercially     ***/
/***     Please, notify me, if you make any changes to this file          ***/
/****************************************************************************/

#include "st0rm.h"
#include "memory.h"
#include "gensnd.h"
#include "genio.h"
#include "m68misc.h"
#include "m68k.h"



static void Z80ErrorMem (u_long addr) {

  PrintELog ("Z80 Error Addr %lx", addr);
  //ExitNow ();
}

/****************************************************************************/
/* Input a byte from given I/O port                                         */
/****************************************************************************/
/*byte Z80_In (byte Port);*/
/* cant handle 16 bit port addresses yet */
//#define Z80_In(Port) ((byte)cpu_readport((Port)&0xff))
#define Z80_In(Port) ((u_char)Z80Read((Port)&0xff,0))

/****************************************************************************/
/* Output a byte to given I/O port                                          */
/****************************************************************************/
/*void Z80_Out (byte Port,byte Value);*/
/* cant handle 16 bit port addresses yet */
//#define Z80_Out(Port,Value) (cpu_writeport((Port)&0xff,Value))
#define Z80_Out(Port,Value) (Z80Write((Port)&0xff,0,Value))

/****************************************************************************/
/* Read a byte from given memory location                                   */
/****************************************************************************/
/*unsigned Z80_RDMEM(dword A);*/
/* ASG 971005 -- changed to cpu_readmem16/cpu_writemem16 */

//#define Z80_RDMEM(A) ((unsigned)cpu_readmem16(A))

static u_char Z80_RDMEM (u_long addr) {

  u_long        m68k_addr;

  addr &= 0xffff;

  if (addr <= 0x2000)
        return ((u_char)mem.ram.z80_ram[addr]);

  if (addr >= 0x4000 && addr <= 0x4004)
        return ((u_char)FMRead (addr, 0));

  if (addr >= 0x6000 && addr <= 0x6010) {
        Z80ErrorMem (addr);
        return 0;
  }

  if (addr == 0x7f11)
        return ((u_char)PSGRead());

  if (addr >= 0x8000) {
        m68k_addr = (io.m68k_addr+(addr-0x8000));
        /*if (m68k_addr < 0x3fffff)
                return (*(mem.rom_space+m68k_addr));
        else if (m68k_addr >= 0xff0000)
                return (*(mem.ram.ram_space+(m68k_addr-0xff0000)));
        else {
                Z80ErrorMem (m68k_addr);
                return 0;
        }*/

        return (get_byte(m68k_addr));
  }

  Z80ErrorMem (addr);
  return 0;
}


/****************************************************************************/
/* Write a byte to given memory location                                    */
/****************************************************************************/
/*void Z80_WRMEM(dword A,byte V);*/
//#define Z80_WRMEM(A,V) (cpu_writemem16(A,V))

static void Z80_WRMEM (u_long addr, u_char to_write) {

  u_long                m68k_addr;
  static u_long         cnt_bank        = 0;

  addr &= 0xffff;

  if (addr <= 0x2000) {
        mem.ram.z80_ram[addr] = to_write;
        return;
  }

  if (addr >= 0x4000 && addr <= 0x4004) {
        FMWrite (addr, 0, (u_long)to_write);
        return;
  }

  if (addr == 0x6000) {

        if (!cnt_bank) io.m68k_addr = 0;

        io.m68k_addr |= (u_long)((to_write&1)<<(cnt_bank+15));

        if (cnt_bank++ == 8) cnt_bank = 0;

        io.m68k_addr &= 0xffffff;

        //PrintELog ("[Z80] New 68K addr = %lx, wrote %lx to %lx", io.m68k_addr, to_write, addr);

        return;
   }

  if (addr == 0x7f11) {
        PSGWrite ((u_long)to_write);
        return;
  }

  if (addr >= 0x8000) {
        m68k_addr = (io.m68k_addr+(addr-0x8000));
        /*if (m68k_addr < 0x3fffff)
                *(mem.rom_space+m68k_addr) = to_write;
        else if (m68k_addr >= 0xff0000)
                *(mem.ram.ram_space+(m68k_addr-0xff0000)) = to_write;
        else
                Z80ErrorMem (m68k_addr);
        return;*/

        put_byte (m68k_addr, (u_char)to_write);
  }

  Z80ErrorMem (addr);
  return;
}

/****************************************************************************/
/* Just to show you can actually use macros as well                         */
/****************************************************************************/
/*
 extern byte *ReadPage[256];
 extern byte *WritePage[256];
 #define Z80_RDMEM(a) ReadPage[(a)>>8][(a)&0xFF]
 #define Z80_WRMEM(a,v) WritePage[(a)>>8][(a)&0xFF]=v
*/

/****************************************************************************/
/* Z80_RDOP() is identical to Z80_RDMEM() except it is used for reading     */
/* opcodes. In case of system with memory mapped I/O, this function can be  */
/* used to greatly speed up emulation                                       */
/****************************************************************************/
/*#define Z80_RDOP(A)           Z80_RDMEM(A)*/
//#define Z80_RDOP(A) ((unsigned)cpu_readop(A))

//#define Z80_RDOP(A) Z80_RDMEM(A)
#define Z80_RDOP(A) mem.ram.z80_ram[A]

/****************************************************************************/
/* Z80_RDOP_ARG() is identical to Z80_RDOP() except it is used for reading  */
/* opcode arguments. This difference can be used to support systems that    */
/* use different encoding mechanisms for opcodes and opcode arguments       */
/****************************************************************************/
/*#define Z80_RDOP_ARG(A)		Z80_RDOP(A)*/
//#define Z80_RDOP_ARG(A) ((unsigned)cpu_readop_arg(A))

#define Z80_RDOP_ARG(A) mem.ram.z80_ram[A]

/****************************************************************************/
/* Z80_RDSTACK() is identical to Z80_RDMEM() except it is used for reading  */
/* stack variables. In case of system with memory mapped I/O, this function */
/* can be used to slightly speed up emulation                               */
/****************************************************************************/
/*#define Z80_RDSTACK(A)		Z80_RDMEM(A)*/
/* ASG 971005 -- changed to cpu_readmem16/cpu_writemem16 */
//#define Z80_RDSTACK(A) ((unsigned)cpu_readmem16(A))

//#define Z80_RDSTACK(A) Z80_RDMEM(A)
#define Z80_RDSTACK(A) mem.ram.z80_ram[A]

/****************************************************************************/
/* Z80_WRSTACK() is identical to Z80_WRMEM() except it is used for writing  */
/* stack variables. In case of system with memory mapped I/O, this function */
/* can be used to slightly speed up emulation                               */
/****************************************************************************/
/*#define Z80_WRSTACK(A,V)	Z80_WRMEM(A,V)*/
//#define Z80_WRSTACK(A,V) (cpu_writemem16(A,V))

//#define Z80_WRSTACK(A,V) Z80_WRMEM(A,V)
#define Z80_WRSTACK(A,V) mem.ram.z80_ram[A]=V
